/*
 -- ============================================================================
 -- FILE NAME	: gpio.v
 -- DESCRIPTION :  General Purpose I/O
 -- ----------------------------------------------------------------------------
 -- Revision  Date		  Coding_by	 Comment
 -- 1.0.0	  2011/06/27  suito		 VK쐬
 -- ============================================================================
*/

/********** ʃwb_t@C **********/
`include "nettype.h"
`include "stddef.h"
`include "global_config.h"

/********** ʃwb_t@C **********/
`include "gpio.h"

/********** W[ **********/
module gpio (
	/********** NbN & Zbg **********/
	input  wire						clk,	 // NbN
	input  wire						reset,	 // Zbg
	/********** oXC^tF[X **********/
	input  wire						cs_,	 // `bvZNg
	input  wire						as_,	 // AhXXg[u
	input  wire						rw,		 // Read / Write
	input  wire [`GpioAddrBus]		addr,	 // AhX
	input  wire [`WordDataBus]		wr_data, // ݃f[^
	output reg	[`WordDataBus]		rd_data, // ǂݏof[^
	output reg						rdy_	 // fB
	/********** ėpo̓|[g **********/
`ifdef GPIO_IN_CH	 // ̓|[g̎
	, input wire [`GPIO_IN_CH-1:0]	gpio_in	 // ̓|[gi䃌WX^0j
`endif
`ifdef GPIO_OUT_CH	 // o̓|[g̎
	, output reg [`GPIO_OUT_CH-1:0] gpio_out // o̓|[gi䃌WX^1j
`endif
`ifdef GPIO_IO_CH	 // o̓|[g̎
	, inout wire [`GPIO_IO_CH-1:0]	gpio_io	 // o̓|[gi䃌WX^2j
`endif
);

`ifdef GPIO_IO_CH	 // o̓|[g̐
	/********** o͐M **********/
	wire [`GPIO_IO_CH-1:0]			io_in;	 // ̓f[^
	reg	 [`GPIO_IO_CH-1:0]			io_out;	 // o̓f[^
	reg	 [`GPIO_IO_CH-1:0]			io_dir;	 // o͕i䃌WX^3j
	reg	 [`GPIO_IO_CH-1:0]			io;		 // o
	integer							i;		 // Ce[^
   
	/********** o͐M̌p **********/
	assign io_in	   = gpio_io;			 // ̓f[^
	assign gpio_io	   = io;				 // o

	/********** o͕̐ **********/
	always @(*) begin
		for (i = 0; i < `GPIO_IO_CH; i = i + 1) begin : IO_DIR
			io[i] = (io_dir[i] == `GPIO_DIR_IN) ? 1'bz : io_out[i];
		end
	end

`endif
   
	/********** GPIO̐ **********/
	always @(posedge clk or `RESET_EDGE reset) begin
		if (reset == `RESET_ENABLE) begin
			/* 񓯊Zbg */
			rd_data	 <= #1 `WORD_DATA_W'h0;
			rdy_	 <= #1 `DISABLE_;
`ifdef GPIO_OUT_CH	 // o̓|[g̃Zbg
			gpio_out <= #1 {`GPIO_OUT_CH{`LOW}};
`endif
`ifdef GPIO_IO_CH	 // o̓|[g̃Zbg
			io_out	 <= #1 {`GPIO_IO_CH{`LOW}};
			io_dir	 <= #1 {`GPIO_IO_CH{`GPIO_DIR_IN}};
`endif
		end else begin
			/* fB̐ */
			if ((cs_ == `ENABLE_) && (as_ == `ENABLE_)) begin
				rdy_	 <= #1 `ENABLE_;
			end else begin
				rdy_	 <= #1 `DISABLE_;
			end 
			/* ǂݏoANZX */
			if ((cs_ == `ENABLE_) && (as_ == `ENABLE_) && (rw == `READ)) begin
				case (addr)
`ifdef GPIO_IN_CH	// ̓|[g̓ǂݏo
					`GPIO_ADDR_IN_DATA	: begin // 䃌WX^ 0
						rd_data	 <= #1 {{`WORD_DATA_W-`GPIO_IN_CH{1'b0}}, 
										gpio_in};
					end
`endif
`ifdef GPIO_OUT_CH	// o̓|[g̓ǂݏo
					`GPIO_ADDR_OUT_DATA : begin // 䃌WX^ 1
						rd_data	 <= #1 {{`WORD_DATA_W-`GPIO_OUT_CH{1'b0}}, 
										gpio_out};
					end
`endif
`ifdef GPIO_IO_CH	// o̓|[g̓ǂݏo
					`GPIO_ADDR_IO_DATA	: begin // 䃌WX^ 2
						rd_data	 <= #1 {{`WORD_DATA_W-`GPIO_IO_CH{1'b0}}, 
										io_in};
					 end
					`GPIO_ADDR_IO_DIR	: begin // 䃌WX^ 3
						rd_data	 <= #1 {{`WORD_DATA_W-`GPIO_IO_CH{1'b0}}, 
										io_dir};
					end
`endif
				endcase
			end else begin
				rd_data	 <= #1 `WORD_DATA_W'h0;
			end
			/* ݃ANZX */
			if ((cs_ == `ENABLE_) && (as_ == `ENABLE_) && (rw == `WRITE)) begin
				case (addr)
`ifdef GPIO_OUT_CH	// o̓|[gւ̏
					`GPIO_ADDR_OUT_DATA : begin // 䃌WX^ 1
						gpio_out <= #1 wr_data[`GPIO_OUT_CH-1:0];
					end
`endif
`ifdef GPIO_IO_CH	// o̓|[gւ̏
					`GPIO_ADDR_IO_DATA	: begin // 䃌WX^ 2
						io_out	 <= #1 wr_data[`GPIO_IO_CH-1:0];
					 end
					`GPIO_ADDR_IO_DIR	: begin // 䃌WX^ 3
						io_dir	 <= #1 wr_data[`GPIO_IO_CH-1:0];
					end
`endif
				endcase
			end
		end
	end

endmodule
